// Copyright (c) Microsoft Corporation. All rights reserved.
// Licensed under the MIT license.
'use strict';
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.deactivate = exports.getHdfsFileAsString = exports.activate = void 0;
const path = require("path");
const vscode = require("vscode");
const azdata = require("azdata");
const util_1 = require("util");
const common_backend_1 = require("common-backend");
let current = undefined;
function activate(context) {
    context.subscriptions.push(vscode.commands.registerCommand('sanddance.view', (commandContext) => {
        if (!commandContext) {
            vscode.window.showErrorMessage('No file was specified for the View in SandDance command');
            return;
        }
        if (commandContext instanceof vscode.Uri) {
            viewFileUriInSandDance(commandContext, context);
        }
        else if (commandContext.nodeInfo) {
            // This is a call from the object explorer right-click.
            downloadAndViewInSandDance(commandContext, context);
        }
    }));
    //make the visualizer icon visible
    vscode.commands.executeCommand('setContext', 'showVisualizer', true);
    // Ideally would unregister listener on deactivate, but this is currently a void function.
    // Issue #6374 created in ADS repository to track this ask
    azdata.queryeditor.registerQueryEventListener({
        onQueryEvent(type, document, args) {
            return __awaiter(this, void 0, void 0, function* () {
                if (type === 'visualize') {
                    const providerid = document.providerId;
                    const provider = azdata.dataprotocol.getProvider(providerid, azdata.DataProviderType.QueryProvider);
                    const data = yield provider.getQueryRows({
                        ownerUri: document.uri,
                        batchIndex: args.batchId,
                        resultSetIndex: args.id,
                        rowsStartIndex: 0,
                        rowsCount: args.rowCount,
                    });
                    const rows = data.resultSubset.rows;
                    const columns = args.columnInfo;
                    const rowsCount = args.rowCount;
                    // Create Json
                    const jsonArray = [];
                    for (let row = 0; row < rowsCount; row++) {
                        const jsonObject = {};
                        for (let col = 0; col < columns.length; col++) {
                            if (!rows[row][col].isNull) {
                                jsonObject[columns[col].columnName] = rows[row][col].displayValue;
                            }
                            // If display value is null, don't do anything for now
                        }
                        jsonArray.push(jsonObject);
                    }
                    viewInSandDance(() => {
                        return new Promise(resolve => resolve(JSON.stringify(jsonArray)));
                    }, document.uri, 'json', context);
                }
            });
        },
    });
}
exports.activate = activate;
function downloadAndViewInSandDance(commandContext, context) {
    return __awaiter(this, void 0, void 0, function* () {
        try {
            const file = yield getHdfsFileAsString(commandContext);
            if (file) {
                const { contents } = file;
                viewInSandDance(() => new Promise(resolve => resolve(contents)), file.fsUriPath, path.extname(file.fsUriPath).substring(1), context);
            }
        }
        catch (e) {
            const error = e;
            vscode.window.showErrorMessage(`Error viewing in sanddance: ${error.message ? error.message : error}`);
        }
    });
}
function viewInSandDance(rawTextPromise, uriFsPath, type, context, uriTabName) {
    const columnToShowIn = vscode.window.activeTextEditor ? vscode.window.activeTextEditor.viewColumn : undefined;
    //only allow one SandDance at a time
    if (current && current.uriFsPath !== uriFsPath) {
        current.panel.dispose();
        current = undefined;
    }
    if (current) {
        //TODO: registerWebviewPanelSerializer to hydrate state
        // If we already have a panel, show it in the target column
        current.panel.reveal(columnToShowIn);
    }
    else {
        // Otherwise, create a new panel
        current = (0, common_backend_1.newPanel)(context, uriFsPath, uriTabName);
        current.panel.onDidDispose(() => {
            current = undefined;
        }, null, context.subscriptions);
        // Handle messages from the webview
        current.panel.webview.onDidReceiveMessage(message => {
            switch (message.command) {
                case 'getFileContent': {
                    rawTextPromise().then(rawText => {
                        if (current && current.panel.visible) {
                            const dataFile = {
                                type,
                                rawText,
                            };
                            const compactUI = context.globalState.get('compactUI');
                            current.panel.webview.postMessage({ command: 'gotFileContent', dataFile, compactUI });
                        }
                    });
                    break;
                }
                case 'setCompactUI': {
                    context.globalState.update('compactUI', message.compactUI);
                    break;
                }
            }
        }, undefined, context.subscriptions);
    }
}
function viewFileUriInSandDance(fileUri, context, uriTabName) {
    const p = () => new Promise(resolve => {
        vscode.workspace.fs.readFile(fileUri).then(uint8array => {
            resolve(new util_1.TextDecoder().decode(uint8array));
        });
    });
    viewInSandDance(p, fileUri.fsPath, path.extname(fileUri.fsPath).substring(1), context, uriTabName);
}
function getHdfsFileAsString(commandContext) {
    return __awaiter(this, void 0, void 0, function* () {
        const extension = vscode.extensions.getExtension('Microsoft.mssql');
        if (extension) {
            const extensionApi = extension.exports;
            const browser = extensionApi.getMssqlObjectExplorerBrowser();
            const node = yield browser.getNode(commandContext);
            const contents = yield node.getFileContentsAsString();
            if (contents !== undefined) {
                return {
                    contents,
                    fsUriPath: node.getNodeInfo().label,
                };
            }
        }
    });
}
exports.getHdfsFileAsString = getHdfsFileAsString;
function deactivate() {
    vscode.commands.executeCommand('setContext', 'showVisualizer', false);
}
exports.deactivate = deactivate;
//# sourceMappingURL=extension.js.map